import os
from urllib.request import urlopen

import numpy as np
import PIL.Image
import requests
from tqdm import tqdm


def load_image_file(file, mode='RGB'):
    im = PIL.Image.open(file)
    if mode:
        im = im.convert(mode)
    return np.array(im)


class Downloader(object):
    def __init__(self):
        pass

    def download(self, url, target):
        # check if target folder exists
        if not os.path.isdir(target):
            os.makedirs(target)
        filename = url.split('/')[-1]
        file_size = int(urlopen(url).info().get('Content-Length', -1))
        chunk_size = 1024
        dest = os.path.join(target, filename)
        if os.path.exists(dest):
            first_byte = os.path.getsize(dest)
        else:
            first_byte = 0
        if first_byte >= file_size:
            return file_size
        header = {"Range": "bytes=%s-%s" % (first_byte, file_size)}
        pbar = tqdm(
            total=file_size,
            initial=first_byte,
            unit='B',
            unit_scale=True,
            desc=filename)
        req = requests.get(url, headers=header, stream=True)
        with open(dest, 'ab') as f:
            for chunk in req.iter_content(chunk_size=chunk_size):
                if chunk:
                    f.write(chunk)
                    pbar.update(1024)
        pbar.close()
        return file_size



import json


def is_jsonable(x):
    try:
        json.dumps(x)
        return True
    except Exception as ex:
        print(ex)
    except:
        return False


class Bundle(object):
    __SOLVERS__ = []

    def add_solver(self, solver):
        self.__SOLVERS__.append(solver)

    @classmethod
    def members(self):
        results = {}
        for attr in dir(self):
            if not attr.startswith("__"):
                if is_jsonable(getattr(self, attr)):
                    results[attr] = getattr(self, attr)
        print(results)
        return results



import json

import toml

from cvpm.bundle import Bundle
from cvpm.server import run_server
from cvpm.utility import Downloader


class Solver(object):
    def __init__(self, toml_file=None):
        self._isReady = False
        self.bundle = None
        self._enable_train = False
        if toml_file is None:
            toml_file = "./pretrained/pretrained.toml"
        self._prepare_models(toml_file)

    @property
    def enable_train(self):
        return self._enable_train

    @property
    def is_ready(self):
        return self._isReady

    @property
    def help_message(self):
        if self.is_ready:
            members = self.bundle.members()
            return json.dumps(members)
        else:
            return json.dumps({"error": "Failed to start", "code": "101"}), 101

    def _prepare_models(self, toml_file):
        parsed_toml = toml.load(toml_file)
        downloader = Downloader()
        if "models" in parsed_toml.keys():
            for each in parsed_toml["models"]:
                downloader.download(each["url"], "pretrained")

    def set_ready(self):
        self._isReady = True

    def set_bundle(self, bundle):
        if issubclass(bundle, Bundle):
            self.bundle = bundle
            solver = self
            bundle.add_solver(self=bundle, solver=solver)

    def infer(self, input, config):
        pass

    def train(self, train_x, train_y, **kwargs):
        pass

    def start(self, port=None):
        print('Server will run on port: ' + str(port))
        run_server(self, port)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cvpm",
    version="0.0.1",
    author="Xiaozhe Yao",
    author_email="xiaozhe.yaoi@gmail.com",
    description="Computer Vision Package Manager",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/unarxiv/cvpm",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Image Recognition",
        "Topic :: Software Development",
    ],
    install_requires=[
        "flask",
        "tqdm",
        "toml",
        "requests",
        "pillow",
        "numpy",
        "gevent",
        "flask-cors",
    ],
)


import json
import logging
import os
import socket
import traceback
import gevent.pywsgi
from flask import Flask, request
from werkzeug.datastructures import ImmutableMultiDict
from werkzeug.utils import secure_filename

# extensions
from flask_cors import CORS

logger = logging.getLogger()
logger.setLevel("INFO")

server = Flask(__name__)
CORS(server)

ALLOWED_EXTENSIONS_TRAIN = set(['zip'])
ALLOWED_EXTENSIONS_INFER = set(['jpg', 'jpeg', 'png'])
UPLOAD_FOLDER = './temp'

server.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER

def str2bool(v):
    return str(v).lower() in ("true", "false", "yes","t","1")

def _isPortOpen(port):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    result = s.connect_ex(('127.0.0.1', port))
    s.close()
    if result == 0:
        return True
    else:
        return False

def get_available_port(start=8080):
    port = start
    while True:
        if _isPortOpen(port):
            port = port + 1
        else:
            break
    return port


def allowed_file(filename, phase):
    ALLOWED_EXTENSIONS = ALLOWED_EXTENSIONS_TRAIN
    if phase == 'infer':
        ALLOWED_EXTENSIONS = ALLOWED_EXTENSIONS_INFER
    return '.' in filename and \
        filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS


@server.route("/")
def help():
    help_message = server.solver.help_message
    return help_message


@server.route("/infer", methods=["GET", "POST"])
def infer():
    if request.method == 'POST':
        results = {}
        config = ImmutableMultiDict(request.form)
        config = config.to_dict()
        if 'file' not in request.files:
            return json.dumps({"error": "no file part!", "code": "400"}), 400
        file = request.files['file']
        if file and allowed_file(file.filename, 'infer'):
            filename = secure_filename(file.filename)
            # make sure the UPLOAD_FOLDER exists
            if not os.path.isdir(server.config['UPLOAD_FOLDER']):
                os.makedirs(server.config['UPLOAD_FOLDER'])
            file_abs_path = os.path.join(server.config['UPLOAD_FOLDER'],
                                         filename)
            file.save(file_abs_path)
            try:
                results = server.solver.infer(file_abs_path, config)
                if str2bool(config['delete_after_process']):
                    os.remove(file_abs_path)
                return json.dumps(results), 200
            except Exception as e:
                traceback.print_exc()
                return json.dumps({"error": str(e), "code": "500"}), 500
        else:
            return json.dumps({
                "error": "Forbidden Filename!",
                "code": "400"
            }), 400


@server.route("/train", methods=["GET", "POST"])
def train():
    if server.solver.enable_train:
        pass
    else:
        return json.dumps({"error": "not supported!", "code": "404"}), 404


def run_server(solver, port=None):
    if port is None:
        port = get_available_port()
    logger.info("Server Running On: " + str(port))
    with server.app_context():
        server.solver = solver
    gevent_server = gevent.pywsgi.WSGIServer(('', port), server)
    gevent_server.serve_forever()
